// Activating 5 relay from the cloud
// Reporting cloud pressed a button
// PWM light intensity control of a led by the cloud
// Display on the cloud of temperature and humidity values

#include <ESP8266WiFi.h>
#include <MQTT.h>
// #include <EEPROM.h>
#include <DHT.h>

//#define DEBUG // uncomment to debug


#define DHT_Pin 13
DHT dht(DHT_Pin, DHT11, 15); // Define DHT11 sensor

long previousMillisDHT = 0;
long TimeDHT = 30000; // 30 seconds of waiting for the temperature and humidity sampling

int oldTemp;
int oldHum;

#define AP_SSID     " *** "
#define AP_PASSWORD " *** "


#define EIOTCLOUD_USERNAME " *** "
#define EIOTCLOUD_PASSWORD " *** "

// creazione oggetto MQTT
#define EIOT_CLOUD_ADDRESS "cloud.iot-playground.com"
MQTT myMqtt("", EIOT_CLOUD_ADDRESS, 1883);


// Definition of used pins
#define GPIO_D0 16
#define GPIO_D1 5
#define GPIO_D2 4
#define GPIO_D3 0
#define GPIO_D4 2
#define GPIO_D8 15
#define pinPotenziometro A0
#define GPIO_D6 12 // pulsante

boolean switchState_1 = LOW;
boolean switchState_2 = LOW;
boolean switchState_3 = LOW;
boolean switchState_4 = LOW;
boolean switchState_5 = LOW;

boolean switchState_dimmer = LOW;
int dimmer = 0;


boolean reading;
boolean old_reading;

String instanceId = " *** ";
boolean result;
String topic("");
String valueStr("");

// Variables used to track the state of outputs
// int RelayStatus = 0;
// int STATUS_EEADR = 20;


void setup()  //*******************************************************************************
{

  // EEPROM.write(STATUS_EEADR, RelayStatus); // It resets the byte which stores the output status (RelayStatus = 0)

  dht.begin();

  oldTemp = -1;
  oldHum = -1;



  pinMode(GPIO_D0, OUTPUT);
  pinMode(GPIO_D1, OUTPUT);
  pinMode(GPIO_D2, OUTPUT);
  pinMode(GPIO_D3, OUTPUT);
  pinMode(GPIO_D4, OUTPUT);
  pinMode(GPIO_D8, OUTPUT);

  digitalWrite(GPIO_D0, HIGH);
  digitalWrite(GPIO_D1, HIGH);
  digitalWrite(GPIO_D2, HIGH);
  digitalWrite(GPIO_D3, HIGH);
  digitalWrite(GPIO_D4, HIGH);
  digitalWrite(GPIO_D8, LOW);

  pinMode(GPIO_D6, INPUT);


#ifdef DEBUG
  Serial.begin(115200);
#endif

  delay(1000);

#ifdef DEBUG
  Serial.println();
  Serial.println();
  Serial.print("Access to the WiFi network: ");
  Serial.println(AP_SSID);
#endif

  WiFi.begin(AP_SSID, AP_PASSWORD);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
#ifdef DEBUG
    Serial.print(".");
#endif
  }

#ifdef DEBUG
  Serial.println("");
  Serial.println("WiFi connection established");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println("Connection to MQTT server");
#endif

  // Generation client Id based on to the MAC address and the last 8 bits of the counter in microseconds
  String clientName;
  //clientName += "esp8266-";
  uint8_t mac[6];
  WiFi.macAddress(mac);
  clientName += macToStr(mac);
  clientName += "-";
  clientName += String(micros() & 0xff, 16);
  myMqtt.setClientId((char*) clientName.c_str());

#ifdef DEBUG
  Serial.print("MQTT client Id:");
  Serial.println(clientName);
#endif

  // setup callbacks
  myMqtt.onConnected(myConnectedCb);
  myMqtt.onDisconnected(myDisconnectedCb);
  myMqtt.onPublished(myPublishedCb);
  myMqtt.onData(myDataCb);
  myMqtt.setUserPwd(EIOTCLOUD_USERNAME, EIOTCLOUD_PASSWORD);
  myMqtt.connect();

  delay(500);

  //setting to OFF state of switches, dimmer and publication
  valueStr = "0";
  topic  = "/Db/" + instanceId + "/1/Sensor.Parameter1";
  result = myMqtt.publish(topic, valueStr);
  topic  = "/Db/" + instanceId + "/2/Sensor.Parameter1";
  result = myMqtt.publish(topic, valueStr);
  topic  = "/Db/" + instanceId + "/3/Sensor.Parameter1";
  result = myMqtt.publish(topic, valueStr);
  topic  = "/Db/" + instanceId + "/4/Sensor.Parameter1";
  result = myMqtt.publish(topic, valueStr);
  topic  = "/Db/" + instanceId + "/5/Sensor.Parameter1";
  result = myMqtt.publish(topic, valueStr);

  topic  = "/Db/" + instanceId + "/7/Sensor.Parameter1";
  result = myMqtt.publish(topic, valueStr);
  topic  = "/Db/" + instanceId + "/7/Sensor.Parameter2";
  result = myMqtt.publish(topic, valueStr);



} // fine setup *****************************************************************************************


void loop() {
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
#ifdef DEBUG
    Serial.print(".");
#endif
  }

  int reading = digitalRead(GPIO_D6);

  if (reading != old_reading)
  {
    if (reading == HIGH)
    {
      valueStr = String("1");
      topic  = "/Db/" + instanceId + "/6/Sensor.Parameter1";
      result = myMqtt.publish(topic, valueStr);
    }
    else
    {
      valueStr = String("0");
      topic  = "/Db/" + instanceId + "/6/Sensor.Parameter1";
      result = myMqtt.publish(topic, valueStr);
    }
    delay(200);
    old_reading = reading;
  }



  unsigned long currentMillisDHT = millis();
  if (currentMillisDHT - previousMillisDHT > TimeDHT)
  {
    previousMillisDHT = currentMillisDHT;

    int hum;               // Humidity value read by the sensor DHT11
    int temp;              // Temperature value read by the sensor DHT11

    // Reading data from the sensor DHT11
    // Value percentage of humidity
    hum = dht.readHumidity();
    // Temperature in degrees Celsius
    temp = dht.readTemperature();


    // Check if the readings are correct
    if (isnan(hum) || isnan(temp)) {
#ifdef DEBUG
      Serial.println("************************************************************************");
      Serial.println("DHT sensor error");
#endif
    }
    else {

#ifdef DEBUG
      Serial.println("************************************************************************");
      // Visualizzazione valori letti
      Serial.print("Humidity: ");
      Serial.print(hum);
      Serial.print("%");
      Serial.print("  -  Temperature: ");
      Serial.print(temp);
      Serial.println("*C");
#endif

      if (temp != oldTemp && temp > 0 && temp < 40)
      {
        valueStr = String(temp);
        topic  = "/Db/" + instanceId + "/10/Sensor.Parameter1";

        result = myMqtt.publish(topic, valueStr);
        oldTemp = temp;
      }

      if (hum != oldHum && hum > 10 && hum < 90)
      {
        valueStr = String(hum);
        topic  = "/Db/" + instanceId + "/11/Sensor.Parameter1";
        result = myMqtt.publish(topic, valueStr);
        oldHum = hum;
      }
    }
  }
}



String macToStr(const uint8_t* mac)
{
  String result;
  for (int i = 0; i < 6; ++i) {
    result += String(mac[i], 16);
    if (i < 5)
      result += ':';
  }
  return result;
}



void myConnectedCb() {
#ifdef DEBUG
  Serial.println("Connect to MQTT server");
#endif

  myMqtt.subscribe("/Db/" + instanceId + "/1/Sensor.Parameter1");
  myMqtt.subscribe("/Db/" + instanceId + "/2/Sensor.Parameter1");
  myMqtt.subscribe("/Db/" + instanceId + "/3/Sensor.Parameter1");
  myMqtt.subscribe("/Db/" + instanceId + "/4/Sensor.Parameter1");
  myMqtt.subscribe("/Db/" + instanceId + "/5/Sensor.Parameter1");

  myMqtt.subscribe("/Db/" + instanceId + "/7/Sensor.Parameter1");
  myMqtt.subscribe("/Db/" + instanceId + "/7/Sensor.Parameter2");

}

void myDisconnectedCb() {
#ifdef DEBUG
  Serial.println("Disconnected. Trying to reconnect ...");
#endif
  delay(500);
  myMqtt.connect();
}

void myPublishedCb() {
#ifdef DEBUG
  Serial.println("publication OK");
#endif
}

void myDataCb(String& topic, String& data) {

  if (topic ==  String("/Db/" + instanceId + "/1/Sensor.Parameter1"))
  {
    switchState_1 = (data == String("1")) ? true : false;
    digitalWrite(GPIO_D0, !switchState_1);
  }

  if (topic ==  String("/Db/" + instanceId + "/2/Sensor.Parameter1"))
  {
    switchState_2 = (data == String("1")) ? true : false;
    digitalWrite(GPIO_D1, !switchState_2);
  }

  if (topic ==  String("/Db/" + instanceId + "/3/Sensor.Parameter1"))
  {
    switchState_3 = (data == String("1")) ? true : false;
    digitalWrite(GPIO_D2, !switchState_3);
  }

  if (topic ==  String("/Db/" + instanceId + "/4/Sensor.Parameter1"))
  {
    switchState_4 = (data == String("1")) ? true : false;
    digitalWrite(GPIO_D3, !switchState_4);
  }

  if (topic ==  String("/Db/" + instanceId + "/5/Sensor.Parameter1"))
  {
    switchState_5 = (data == String("1")) ? true : false;
    digitalWrite(GPIO_D4, !switchState_5);
  }


  if (topic ==  String("/Db/" + instanceId + "/7/Sensor.Parameter2"))
  {
    if (data == String("1"))
    {
      switchState_dimmer = true;
      analogWrite(GPIO_D8, dimmer);
    }
    else
    {
      switchState_dimmer = false;
      analogWrite(GPIO_D8, 0);
    }
  }

  if (topic ==  String("/Db/" + instanceId + "/7/Sensor.Parameter1"))
  {
    dimmer = map(data.toInt(), 0, 100, 0, 255);

    if ( switchState_dimmer == true)
    {
      analogWrite(GPIO_D8, dimmer);
    }
  }



}


